/************************************************************************************
*************************************************************************************
*http.c: 
*The file makes connection with server using libcurl. Sends data to the server and handles the response
*Copyright (c) 2011 by Manu Dhundi. All rights reserved.
*
*************************************************************************************
************************************************************************************/
#include <stdio.h>      /* fprintf() */
#include <stdlib.h>	  	/* exit() */
#include <string.h>     /* strlen(), strncpy() */

#include <curl/curl.h> 		/* For libcurl functions */

#include "sece_config.h" /* For any config parameters */
#include "http.h"						/* Own header */

#define RECEIVEBUFMAXLEN	4096

static int gPayloadFlag = 1;

/***************************************************************************
* read_callback()
*
*

***************************************************************************/
static size_t read_callback(void *ptr, size_t size, size_t nmemb, void *stream)
{
	size_t retcode = 0;

	if(gPayloadFlag) { /* Note here since our payload is small we are assuming that 
                                 everything is copied in 1 go */
		fprintf(stderr, "size: %d\t nmemb: %d\t stream: %s\n",size,nmemb, (char*)stream);
		strncpy(ptr,stream,size*nmemb);
		retcode = strlen(ptr);
		gPayloadFlag = 0; /* Indicating that copy is over so that on next call 0 is returned */
		return retcode;
	}
	else {/*Zero must be returned indicating end of payload*/
		gPayloadFlag = 1;
		return 0;
	}
}

/***************************************************************************
* write_callback()
*
*

***************************************************************************/
static size_t write_callback(void *ptr, size_t size, size_t nmemb, void *stream)
{
	size_t retcode = 0;
	char *p = (char*)stream;

	if((size*nmemb) >= RECEIVEBUFMAXLEN) {
		fprintf(stderr, "Data overflow at receive !!!\n");
		return 0;
	}
	else {
		strncpy(p,ptr,size*nmemb);
		p[size*nmemb] = 0; /* Null terminating the string */
		//fprintf(stderr, "Received Data:\n%s\n",p);
		retcode = strlen(p);
		return(retcode);
	}
}

/***************************************************************************
* httpInit()
*
*
***************************************************************************/
void httpInit(void)
{
  return;
}


/***************************************************************************
* httpSend()
*
*
***************************************************************************/
int httpSend(sendMethod eSendType, char *pPayLoad)
{
	/* Curl Variables */
	long httpResponseCode = 0;
	CURL *curl;
  CURLcode res;
	struct curl_slist *headers=NULL;
	char cReceiveBuf[RECEIVEBUFMAXLEN] = {0};
	
	/* Check if pPayLoad is valid */
	if(pPayLoad == NULL) {
		fprintf(stderr,"Error in httpSend");
		return -1;
	}
	
	/* In windows, this will init the winsock 
	 curl_global_init(CURL_GLOBAL_ALL); */ 
	
	/* get a curl handle */ 
	curl = curl_easy_init();
	
	if(eHTTP_PUT == eSendType) { /* HTTP PUT */
		if(curl) {
			/* data to be uploaded is provided in read_callback */ 
			curl_easy_setopt(curl, CURLOPT_READFUNCTION, read_callback);
			curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
			
			/*To get the server response*/
			curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_callback);
			curl_easy_setopt(curl, CURLOPT_WRITEDATA, cReceiveBuf);
			
			/* set uploading (HTTP PUT) */ 
			curl_easy_setopt(curl, CURLOPT_UPLOAD, 1L);			
			
			/* Upload URL, this URL is a file name, not a directory */ 
			curl_easy_setopt(curl, CURLOPT_URL, SERVERADDR);

			/* pPayLoad for upload. This is copied in the  read_callback*/
			fprintf(stderr, "PayLoad:%s\n",pPayLoad);
			curl_easy_setopt(curl, CURLOPT_READDATA, pPayLoad);

			/* provide the size of the upload, typecast the value
			to curl_off_t to use the correct data size */ 
			curl_easy_setopt(curl, CURLOPT_INFILESIZE_LARGE,
			(curl_off_t)(strlen(pPayLoad)));

			/* Send Data */ 
			res = curl_easy_perform(curl);
			
			/* Check the server response */
			curl_easy_getinfo (curl, CURLINFO_RESPONSE_CODE, &httpResponseCode);
			
			if (httpResponseCode == 200 && res != CURLE_ABORTED_BY_CALLBACK) {
				fprintf(stderr,"Success from Server\n");
			}
			else {
				fprintf(stderr,"Error %ld from Server\n",httpResponseCode);
			}

			/* always cleanup */ 
			curl_slist_free_all(headers); /* free the header list */
			curl_easy_cleanup(curl);
		}
		else {
			fprintf(stderr, "Failed to get curl handle\n");
			exit(1);
		}
	}
	else {
		if(curl) {
			curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
			/* First set the URL that is about to receive our POST. This URL can
				 just as well be a https:// URL if that is what should receive the
				 data. */ 
			curl_easy_setopt(curl, CURLOPT_URL, SERVERADDR);

			/*To get the server response*/
			curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_callback);
			curl_easy_setopt(curl, CURLOPT_WRITEDATA, cReceiveBuf);
			
			/*Set content type field in the header*/
			headers = curl_slist_append(headers, "Content-Type: application/json");

			curl_easy_setopt(curl, CURLOPT_HTTPHEADER, headers);

			/* Now specify the POST data */ 
			fprintf(stderr, "Payload:%s\n",pPayLoad);
			curl_easy_setopt(curl, CURLOPT_POSTFIELDS, pPayLoad);

			/* Perform the request, res will get the return code */ 
			res = curl_easy_perform(curl);

			/* Check the server response */
			curl_easy_getinfo (curl, CURLINFO_RESPONSE_CODE, &httpResponseCode);
			
			if (httpResponseCode == 200 && res != CURLE_ABORTED_BY_CALLBACK) {
				fprintf(stderr,"Success from Server\n");
			}
			else {
				fprintf(stderr,"Error %ld from Server\n",httpResponseCode);
			}
			
			/* always cleanup */ 
			curl_easy_cleanup(curl);
		}
		else {
			fprintf(stderr, "Failed to get curl handle\n");
			exit(1);
		}
	}
	return 0;
}
