/************************************************************************************
*************************************************************************************
*secs.c: 
*SECE Explorer Communication Setup Module. Handles communication with the server.
*Sends the services to the http.c ( which connects to server) in the desired (HTTP, JSON doc) fashion
*Copyright (c) 2011 by Manu Dhundi. All rights reserved.
*
*************************************************************************************
************************************************************************************/
#include <stdio.h>			/* fprintf() */
#include <string.h>		/* strcpy() */

#include <json/json.h>

#include "sece_config.h" /* For any config parameters */
#include "http.h"				/* httpSend() */
#include "secs.h"				/* own header */
#include "utilities.h"		/* mystrcopy() */

#define PAYLOADBUFSIZE 2048

/***************************************************************************
* secsPreparePayload()
* Prepares the JSON doc for the service discovered
* For now this is capable of handling only 1 service at a time. 
*  
*
***************************************************************************/
static int secsPreparePayload(serviceInfoStruct* pService, int iServiceCount, char* pBuf)
{
	int iCount = 0;
	
	/* JSON variables */
	json_object * jobj = json_object_new_object();
	json_object *jstring_ServiceName;
	json_object *jstring_ServiceType;
	json_object *jstring_ServiceIP;
	json_object *jstring_HostName;
	json_object *jstring_Port;
	json_object *jint_AddServiceFlag; 
	
	/* Check for validity of the function inputs */
	if((pService == NULL) || (pBuf == NULL))
	{
		fprintf(stderr,"Error in secsPreparePayload");
		return -1;
	}
	
	/* Prepare the JSON object */
	jstring_ServiceName = json_object_new_string(pService[iCount].serviceName);
	jstring_ServiceType = json_object_new_string(pService[iCount].serviceType);
	jstring_ServiceIP = json_object_new_string(pService[iCount].serviceIP);
	jstring_HostName = json_object_new_string(pService[iCount].hostName);
	jstring_Port = json_object_new_string(pService[iCount].port);
	jint_AddServiceFlag = json_object_new_int(pService[iCount].serviceAddFlag);
	
  json_object_object_add(jobj,"ServiceName", jstring_ServiceName);
	json_object_object_add(jobj,"ServiceType", jstring_ServiceType);
	json_object_object_add(jobj,"ServiceIP", jstring_ServiceIP);
	json_object_object_add(jobj,"HostName", jstring_HostName);
	json_object_object_add(jobj,"Port", jstring_Port);
	json_object_object_add(jobj,"ServiceAddFlag", jint_AddServiceFlag);
	
	/*Copy the JSON buf to payload */
	strcpy(pBuf,json_object_to_json_string(jobj));
#ifdef NOSERVER
	fprintf (stderr, "The json object created: %s\n",pBuf);
#endif
	return 0;	
}

/***************************************************************************
* secsInit()
*
*
***************************************************************************/
int secsInit(void)
{
	httpInit();
	return 0;
}

/***************************************************************************
* secsSendServices()
* SECE Explorer application makes this call to send the services discovered to the server
*
***************************************************************************/
int secsSendServices(serviceInfoStruct* pService, int iServiceCount)
{
	int iRetVal = 0;
	char cPayLoadBuf[PAYLOADBUFSIZE] = {0};

	/* Check if a service is passed */
	if(pService == NULL) {
		fprintf(stderr,"Error in secsSendList");
		return -1;
	}

	/* Prepare the JSON doc for the service discovered */
	iRetVal = secsPreparePayload(pService, iServiceCount, cPayLoadBuf);
	if(iRetVal != 0) {
		fprintf(stderr,"Error in secsSendList 1");
		return -1;
	}

	/* Send the service discovered to the server. Choose POST or PUT */
	iRetVal = httpSend(eHTTP_POST, cPayLoadBuf);
	if(iRetVal != 0) {
		fprintf(stderr,"Error in secsSendList 2");
		return -1;
	}
	return 0;
}
